/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLModelDocRepository.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser;

import java.util.List;
import java.util.Map;

import org.eclipse.eodm.owl.resource.parser.impl.ReferenceRecorder;

/**
 * OWLModelDocRepository - ODM ( Ontology Definition Meta-Model ) repository.
 * <p>
 * Model URI - Model pairs storage. Documents storage. Model - Document Mappings
 * storage.
 */
public interface OWLModelDocRepository {
    /**
     * Add an OWL Ontology Model.
     * 
     * @param ontoModel
     *            the OWL Ontology Model to be added
     */
    void addOntoModel(OWLOntoModel ontoModel);

    /**
     * Get OWL Ontology Model <code>OWLOntoModel</code> by URI.
     * 
     * @param uri
     *            the model uri
     * @return the ontology model with the uri
     */
    OWLOntoModel getOntoModel(String modelURI);

    /**
     * Get all the OWL Ontology Models.
     * 
     * @return all the ontology models
     */
    Map getAllOntoModels();

    /**
     * Add a mapping between Ontology Model and OWL Documents to the repository.
     * 
     * @param modelDocMapping
     *            the Ontology Model - OWL Document mapping
     */
    void addModelDocMapping(OWLModelDocMapping modelDocMapping);

    /**
     * Get the OWL Ontology Model - OWL Document mapping by model uri.
     * 
     * @param modelURI
     *            the Ontology Model URI
     * @return the mapping between Ontology Mode and OWL Documents
     */
    OWLModelDocMapping getMappingByModelURI(String modelURI);

    /**
     * Get the OWL Ontology Model - OWL Document mapping by document.
     * 
     * @param document
     *            the Document
     * @return the mapping between Ontology Model and OWL Documents
     */
    OWLModelDocMapping getMappingByDocument(OWLDocument document);

    /**
     * Add an OWL Document.
     * 
     * @param document
     *            the OWL Document
     */
    void addDocument(OWLDocument document);

    /**
     * Get OWL Document <code>OWLDocument</code> by local URI.
     * 
     * @param uri
     *            the document local uri
     * @return the owl document with local uri of localURI
     */
    OWLDocument getDocumentByLocalURI(String localURI);

    /**
     * Get OWL Document <code>OWLDocument</code> by public URI.
     * 
     * @param uri
     *            the document public uri
     * @return the owl document with local uri of publicURI
     */
    OWLDocument getDocumentByPublicURI(String publicURI);

    /**
     * Get all the OWL Documents.
     * 
     * @return all the owl documents
     */
    List getAllDocuments();

    /**
     * Add an axiom which relates to RDFSClass.
     * 
     * @param axiom
     *            the axiom which relates to RDFSClass
     */
    void addReference(ReferenceRecorder axiom);

    /**
     * Get all the axioms related to RDFSClass
     * 
     * @return all the axioms related to RDFSClass
     */
    List getReferences();

    /**
     * Clear all the resources stored in the repository. Including all documents
     * and ontology models.
     */
    void clearResources();
}