/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: ECore2OWLTransformer.java,v 1.3 2006/06/22 05:30:48 gxie Exp $
 */

package org.eclipse.eodm.owl.transformer.ecore;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.eodm.owl.CardinalityRestriction;
import org.eclipse.eodm.owl.EnumeratedClass;
import org.eclipse.eodm.owl.Individual;
import org.eclipse.eodm.owl.MaxCardinalityRestriction;
import org.eclipse.eodm.owl.MinCardinalityRestriction;
import org.eclipse.eodm.owl.OWLClass;
import org.eclipse.eodm.owl.OWLDatatypeProperty;
import org.eclipse.eodm.owl.OWLFactory;
import org.eclipse.eodm.owl.OWLObjectProperty;
import org.eclipse.eodm.owl.OWLOntology;
import org.eclipse.eodm.owl.resource.OWLXMLResourceFactoryImpl;
import org.eclipse.eodm.owl.transformer.OWLTransformationException;
import org.eclipse.eodm.rdf.resource.RDFXMLResource;
import org.eclipse.eodm.rdfs.Namespace;
import org.eclipse.eodm.rdfs.Ontology;
import org.eclipse.eodm.rdfs.PlainLiteral;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSClass;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;
import org.eclipse.eodm.rdfs.vocabulary.XSD;

public class ECore2OWLTransformer {

    // input ecore file
    private String inputEcoreFile = null;

    // output owl file
    private String outputOWLFile = null;

    // owl factory
    private OWLFactory factory = null;

    // owl ontology
    private OWLOntology ontology = null;

    // ecore factory
    //private EcoreFactory ecoreFactory = null;

    // ecorepackage
    private EcorePackage ecorePackage = null;

    // ecore package
    //private EPackage epackage = null;

    // resource set
    private ResourceSet resourceSet = null;

    // annoymous Namespace
    private Namespace aNS = null;

    // default Namespace
    private Namespace dNS = null;

    // record log for transformer
    private BufferedWriter bw = null;

    private StringBuffer sb = null;

    public ECore2OWLTransformer()
    {
        initialize( null );
    }
    public ECore2OWLTransformer(String logFile) {
        // initialization
        initialize( logFile );
    }

    /**
     * 
     * Translate function Translate the input ecore file to target owl file.
     * 
     * @param input
     *            input ecore file
     * @throws OWLTransformationException
     */
    public void convertECore2OWL(String input)
            throws OWLTransformationException {
        convertECore2OWL(input, null);
    }

    /**
     * 
     * Translate function Translate the input ecore file to target owl file.
     * 
     * @param input
     *            input ecore file
     * @param output
     *            target owl file
     * @throws OWLTransformationException
     */
    public void convertECore2OWL(String input, String output)
            throws OWLTransformationException {

        checkFiles(input, output);

        // load ROOT EPackage
        resourceSet.getResource(URI.createFileURI(inputEcoreFile), true);

        convertECore2OWL(resourceSet, output);
    }

    /**
     * Translate function Translate the input resource set to target owl file.
     * 
     * @param resourceSet
     *            input ecore model to OWL file
     * @param output
     *            target owl file
     * @throws OWLTransformationException
     * 
     */
    public void convertECore2OWL(ResourceSet resSet, String output)
            throws OWLTransformationException {

        resourceSet = resSet;
        // load ROOT EPackage
        Resource resource = (Resource) resourceSet.getResources().get(0);

        EPackage ep = null;
        if ((resource.getContents().size() == 0)
            || !EPackage.class.isInstance(resource.getContents().get(0))) {
            OWLTransformationException e = new OWLTransformationException(
                    "Input Ecore has no ROOT EPackage : input Ecore model is invalid.");

            // log
            sb
                    .append("Input Ecore has no ROOT EPackage : input Ecore model is invalid.\n\n");
            log(sb.toString());

            throw e;
        } else {
            // TODO1
            ep = (EPackage) resource.getContents().get(0);

            for (Iterator iter = ep.getESubpackages().iterator(); iter
                    .hasNext();) {
                EPackage sube = (EPackage) iter.next();
                ep.getEClassifiers().addAll(sube.getEClassifiers());
            }

            // ep = ecoreFactory.createEPackage();
            Iterator i = resource.getContents().iterator();
            i.next();
            while (i.hasNext()) {
                EPackage e = (EPackage) i.next();

                for (Iterator iter = e.getESubpackages().iterator(); iter
                        .hasNext();) {
                    EPackage sube = (EPackage) iter.next();
                    ep.getEClassifiers().addAll(sube.getEClassifiers());
                }

                ep.getEClassifiers().addAll(e.getEClassifiers());
            }
        }

        try {
            // Ecore -> OWL
            // create ontology from EPackage
            PlainLiteral label = RDFSFactory.eINSTANCE.createPlainLiteral();
            label.setLexicalForm("EPackage_" + ep.getName());
            ontology.getRDFSLabel().add(label);
            ontology.setLocalName(ep.getName());
            ontology.setNamespace(getEObjectNS(ep));
            dNS = getEObjectNS(ep);

            createComments(ontology, ep, ontology);
            Namespace xsd = null;

            // EClass -> OWLClass
            for (Iterator iter = ep.getEClassifiers().iterator(); iter
                    .hasNext();) {

                EClassifier classifier = (EClassifier) iter.next();
                transformEClass(classifier);

            }

            // eSuperTypes -> RDFSSubClassOf
            for (Iterator iter = ep.getEClassifiers().iterator(); iter
                    .hasNext();) {
                EClassifier classifier = (EClassifier) iter.next();
                if (classifier instanceof EClass) {
                    EClass eclass = (EClass) classifier;
                    Iterator sulist = eclass.getESuperTypes().iterator();
                    // OWLClass oclass = (OWLClass)
                    // ontology.getContainedResource(null, prefixName(eclass));
                    OWLClass oclass = (OWLClass) (OWLClass) getResource(eclass);

                    if (oclass == null)
                        continue;

                    while (sulist.hasNext()) {
                        EClassifier temps = (EClassifier) sulist.next();
                        if (temps instanceof EClass) {
                            // TODO1
                            OWLClass father = (OWLClass) getResource(temps);
                            if (father == null)
                                continue;
                            oclass.getRDFSSubClassOf().add(father);

                            // log
                            sb
                                    .append("eSuperTypes("
                                            + temps.getName()
                                            + ", "
                                            + eclass.getName()
                                            + "transformed to RDFSSubClassOf successfully!\n");
                        }
                    }
                }
            }

            // EAttribute -> OWLDatatypeProperty
            // EReference -> OWLObjectProperty
            for (Iterator iter = ep.getEClassifiers().iterator(); iter
                    .hasNext();) {
                EClassifier classifier = (EClassifier) iter.next();
                if (classifier instanceof EClass) {

                    EClass eclass = (EClass) classifier;
                    // TODO1
                    OWLClass oclass = (OWLClass) getResource(eclass);

                    if (oclass == null)
                        continue;

                    // EAttribute -> OWLDatatypeProperty
                    Iterator i = eclass.getEAttributes().iterator();
                    while (i.hasNext()) {
                        boolean existingProperty = false;
                        EAttribute attribute = (EAttribute) i.next();

                        OWLDatatypeProperty odp = null;
                        RDFSResource val = getResource(attribute);

                        if (val == null) {
                            // New property
                            odp = factory.createOWLDatatypeProperty();

                            odp.setLocalName(getUniquName(odp, attribute
                                    .getName()));
                            odp.setNamespace(getEObjectNS(attribute));

                            existingProperty = true;

                            ontology.getContains().add(odp);
                            addResource(attribute, odp);
                            addResource(odp.getLocalName(), odp); // add name

                            createComments(ontology, attribute, odp);
                        } else {
                            odp = (OWLDatatypeProperty) val;
                        }

                        // set domain & range
                        odp.getRDFSDomain().add(oclass);

                        if (!EEnum.class.isInstance(attribute.getEType())) {
                            // create range as a datatype instance, including
                            // XSD, EMF and user-defined datatype
                            RDFSDatatype type = RDFSFactory.eINSTANCE
                                    .createRDFSDatatype();
                            // check if XSD datatype
                            if (attribute.getEType() == ecorePackage
                                    .getEBoolean())
                                type.setLocalName(XSD.sxboolean);
                            else if (attribute.getEType() == ecorePackage
                                    .getEFloat())
                                type.setLocalName(XSD.sxfloat);
                            else if (attribute.getEType() == ecorePackage
                                    .getEByte())
                                type.setLocalName(XSD.sxbyte);
                            else if (attribute.getEType() == ecorePackage
                                    .getEInt())
                                type.setLocalName(XSD.sxint);
                            else if (attribute.getEType() == ecorePackage
                                    .getELong())
                                type.setLocalName(XSD.sxlong);
                            else if (attribute.getEType() == ecorePackage
                                    .getEDouble())
                                type.setLocalName(XSD.sxdouble);
                            else if (attribute.getEType() == ecorePackage
                                    .getEShort())
                                type.setLocalName(XSD.sxshort);
                            else if (attribute.getEType() == ecorePackage
                                    .getEIntegerObject())
                                type.setLocalName(XSD.sinteger);
                            else if (attribute.getEType() == ecorePackage
                                    .getEString())
                                type.setLocalName(XSD.sxstring);

                            if (type.getLocalName() != null) {
                                // is XSD type
                                if (xsd == null) {
                                    xsd = RDFSFactory.eINSTANCE
                                            .createNamespace();
                                    xsd.setName(XSD.SHORTNAME);
                                    xsd.setURI(XSD.NAMESPACE);
                                    ontology.getOwnedNamespace().add(xsd);
                                }
                                type.setNamespace(xsd);
                                if (!existingProperty) {
                                    // Create label
                                    label = RDFSFactory.eINSTANCE
                                            .createPlainLiteral();
                                    label.setLexicalForm("EAttribute_"
                                                         + attribute.getName());
                                    odp.getRDFSLabel().add(label);
                                }
                            } else {
                                // user-defined datatype
                                EClassifier ec_temp = attribute.getEType();
                                RDFSDatatype tmp_type = (RDFSDatatype) getResource(ec_temp);

                                if (tmp_type != null) {
                                    type = tmp_type;
                                } else {
                                    type.setLocalName(ec_temp.getName());
                                    type.setNamespace(getEObjectNS(ec_temp));

                                    ontology.getContains().add(type);
                                    addResource(ec_temp, type);
                                }

                            }
                            addRDFSRange(odp, type);

                            // log
                            // sb.append(" The type of OWLDatatypeProperty " +
                            // attribute.getName() + " is " +
                            // type.getLocalName() + "\n");

                        } else {
                            // a oneof attribute
                            // TODO1
                            EClassifier ec_temp = attribute.getEType();
                            EnumeratedClass enumclass = (EnumeratedClass) getResource(ec_temp);
                            // EnumeratedClass enumclass = (EnumeratedClass)
                            // ontology.getContainedResource(null,
                            // prefixName(ec_temp));

                            /*
                             * RDFBag bag = (RDFBag) ontology
                             * .getContainedResource(null, attribute
                             * .getEType().getName());
                             */
                            if (!existingProperty) {
                                label = RDFSFactory.eINSTANCE
                                        .createPlainLiteral();
                                label.setLexicalForm("EAttribute_EEnum_"
                                                     + attribute.getName());
                                odp.getRDFSLabel().add(label);
                            }
                            if (enumclass != null) {
                                RDFSDatatype dummy = RDFSFactory.eINSTANCE
                                        .createRDFSDatatype();
                                dummy.setNamespace(getEObjectNS(ec_temp));
                                dummy.setLocalName("EEnum_"
                                                   + enumclass.getLocalName());
                                dummy.getRDFSIsDefinedBy().add(enumclass);
                                addRDFSRange(odp, dummy);
                                ontology.getContains().add(dummy);

                                // log
                                sb.append("  The type of OWLDatatypeProperty "
                                          + attribute.getName()
                                          + " is OWLEnumeratedClass\n");
                            }
                        }
                    } // End of "EAttribute -> OWLDatatypeProperty"

                    // EReference -> OWLObjectProperty
                    i = eclass.getEReferences().iterator();
                    while (i.hasNext()) {
                        //boolean existingProperty = false;

                        EReference reference = (EReference) i.next();

                        RDFSResource res = getResource(reference);

                        OWLObjectProperty oop = (OWLObjectProperty) res;

                        if (oop == null) {
                            // New RDFProperty
                            oop = factory.createOWLObjectProperty();

                            oop.setLocalName(getUniquName(oop, reference
                                    .getName()));
                            oop.setNamespace(getEObjectNS(reference));

                            ontology.getContains().add(oop);
                            addResource(reference, oop);
                            addResource(oop.getLocalName(), oop);

                            //existingProperty = true;
                            createComments(ontology, reference, oop);
                        }

                        // domain
                        oop.getRDFSDomain().add(oclass);
                        // range
                        EClass ec_temp = reference.getEReferenceType();
                        OWLClass range = (OWLClass) getResource(ec_temp);

                        if (range != null) {
                            addRDFSRange(oop, range);
                        } else {
                            transformEClass(ec_temp);
                        }

                        /*
                         * determine multiplicity: 5 situation 
                         *     situation    lowBound    upperBound 
                         *        0..*          0           -1 
                         *        m..*          m           -1 
                         *        0..n          0           n 
                         *        m..n          m           n 
                         *        m..m          m           m
                         */
                        int lowerBound = reference.getLowerBound();
                        int upperBound = reference.getUpperBound();

                        if (upperBound == -1 && lowerBound > 0) {
                            // minCardinality
                            oclass.getRDFSSubClassOf().add(
                                    createMinCardinarlityRestriction(oop,
                                            lowerBound));
                        } else if (lowerBound == upperBound) {
                            // Cardinality
                            oclass.getRDFSSubClassOf().add(
                                    createCardinarlityRestriction(oop,
                                            lowerBound));
                        } else if (lowerBound > 0) {
                            // minCardinality & maxCardinality
                            oclass.getRDFSSubClassOf().add(
                                    createMinCardinarlityRestriction(oop,
                                            lowerBound));
                            oclass.getRDFSSubClassOf().add(
                                    createMaxCardinarlityRestriction(oop,
                                            upperBound));
                        } else {
                            // maxCardinality
                            oclass.getRDFSSubClassOf().add(
                                    createMaxCardinarlityRestriction(oop,
                                            upperBound));
                        }
                    } // End of "EReference -> OWLObjectProperty"
                } // End of "if (classifier instanceof EClass)"
            }

            // log
            sb.append("\n");

        } catch (Exception e) {
            OWLTransformationException te = new OWLTransformationException(
                    "Ecore to OWL Mapping failed: " + e.toString());

            // log
            sb.append("Ecore to OWL Mapping failed: " + e.toString() + "\n\n");
            log(sb.toString());

            throw te;
        }

        try {
            resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
                    .put(Resource.Factory.Registry.DEFAULT_EXTENSION,
                            new OWLXMLResourceFactoryImpl());
            Resource saver = resourceSet.createResource(URI
                    .createFileURI(outputOWLFile));
            saver.getContents().add(ontology);
            HashMap options = new HashMap();
            options.put(RDFXMLResource.OPTION_ENCODING, "UTF-8");
            saver.save(options);

            // log
            sb.append("Ontology model saved successfully!\n\n");

            // OWLXMLSaver.saveToFile(ontology, outputOWLFile, "UTF-8");
        } catch (Exception e) {
            OWLTransformationException te = new OWLTransformationException(
                    "Error during serialize into owl file: " + e.toString());

            // log
            sb.append("Error during serialize into owl file: "
                      + e.toString() + "\n\n");
            log(sb.toString());

            throw te;
        }

        // logging
        log(sb.toString());

    }

    /*
     * name
     * 
     * @param o @param localName
     */
    private String getUniquName(Object o, String localName) {
        String name = localName;

        // String prefix;
        // if ((o instanceof OWLClass)&& !(o instanceof EnumeratedClass)) {
        // prefix = "C-";
        // } else if (o instanceof EnumeratedClass) {
        // prefix = "E-";
        // } else if (o instanceof Individual) {
        // prefix = "I-";
        // } else if (o instanceof RDFSDatatype) {
        // prefix = "T-";
        // } else if (o instanceof OWLDatatypeProperty) {
        // prefix = "DP-";
        // } else if (o instanceof OWLObjectProperty) {
        // prefix = "OP-";
        // }

        int i = 1;
        while (true) {
            RDFSResource resource = ontology.getContainedResource(null, name);
            if (resource == null) {
                return name;
            }

            name = localName + "_" + i;
            i++;
        }

    }

    /*
     * record log for transformer
     * 
     * @param s
     */
    private void log(String s) {
        if ( bw != null )
        {
        try {
            bw.write(s);
            bw.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        }
    }

    /*
     * initialization
     */
    private void initialize( String logFile ) {
        // create factory
        factory = OWLFactory.eINSTANCE;
        // create ontology
        ontology = factory.createOWLOntology();
        // anonymous namespace
        aNS = RDFSFactory.eINSTANCE.createNamespace();
        // TODO1
        aNS.setName(RDFS.BNODE_SHORTNAME);
        aNS.setURI(RDFS.BNODE_NAMESPACE);
        // System.out.println(ans.getURI());
        ontology.getOwnedNamespace().add(aNS);

        // init ecore package
        //ecoreFactory = EcoreFactory.eINSTANCE;
        ecorePackage = EcorePackage.eINSTANCE;
        //epackage = ecoreFactory.createEPackage();

        // init resource set
        resourceSet = new ResourceSetImpl();
        resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
                .put(Resource.Factory.Registry.DEFAULT_EXTENSION,
                        new XMIResourceFactoryImpl());

        // init buffered writer
        if ( logFile !=null )
        {
            try {
                // TODO
                bw = new BufferedWriter(new FileWriter(logFile));
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else {
            bw = null ;
        }

        sb = new StringBuffer();
    }

    /**
     * Check input and output files
     * 
     * @param input
     *            input ecore file
     * @param output
     *            target owl file
     * @throws OWLTransformationException
     */
    private void checkFiles(String input, String output)
            throws OWLTransformationException {
        // check input params
        if (input == null) {
            OWLTransformationException e = new OWLTransformationException(
                    "Initate mapping failed : input Ecore model is NULL.");
            throw e;
        }

        // check input and output files
        if (input != null)
            if (input.equals("") == false)
                setInputEcoreFile(input);
        if (output != null)
            if (output.equals("") == false)
                setOutputOWLFile(output);
    }

    /**
     * Add range to property without duplication
     * 
     * @param property
     * @param range
     */
    private void addRDFSRange(RDFProperty property, RDFSClass range) {
        if (property == null || range == null)
            return;

        boolean existingRange = false;
        for (Iterator iter = property.getRDFSRange().iterator(); iter.hasNext();) {
            if (((RDFSClass) iter.next()).getURI().equals(range.getURI())) {
                existingRange = true;
                break;
            }
        }

        if (!existingRange) {
            property.getRDFSRange().add(range);
        }
    }

    /**
     * create RDFSComments for RDFSResource
     * 
     * @param onto
     * @param ec
     * @param rs
     */
    private void createComments(Ontology onto, ENamedElement ec, RDFSResource rs) {

        if (onto == null || ec == null || rs == null)
            return;

        Iterator anlist = ec.getEAnnotations().iterator();

        while (anlist.hasNext()) {
            EAnnotation an = (EAnnotation) anlist.next();

            PlainLiteral comment = RDFSFactory.eINSTANCE.createPlainLiteral();
            comment.setLexicalForm(eAnnotation2String(an));
            rs.getRDFSComment().add(comment);
        }

        // add instanceClassName if EDataType
        if (ec instanceof EDataType) {
            PlainLiteral instanceClassName = RDFSFactory.eINSTANCE
                    .createPlainLiteral();
            instanceClassName.setLexicalForm(XSD.ECORE_INSTANCE_CLASS_NAME
                                             + ((EDataType) ec)
                                                     .getInstanceClassName());
            rs.getRDFSComment().add(instanceClassName);
        }

    }

    private String eAnnotation2String(EAnnotation an) {
        String str = an.getSource() + ": ";
        EMap detail = (EMap) an.getDetails();
        for (Iterator iter = detail.keySet().iterator(); iter.hasNext();) {
            Object key = iter.next();
            str += "  " + key + ":" + detail.get(key);
        }
        return replaceKeywords(str);
    }

    private static String replaceKeywords(String str) {
        str = str.replaceAll("&", "&amp;");
        str = str.replaceAll("<", "&lt;");
        str = str.replaceAll(">", "&gt;");
        str = str.replaceAll("'", "&apos;");
        str = str.replaceAll("\"", "&quot;");

        return str;
    }

    /**
     * set input file URL. If outputfile's URL is "null", automaticly generate a
     * .owl file in the same folder as default output file.
     * 
     * @param in
     *            input file's URL
     */
    public void setInputEcoreFile(String in) {

        if (in == null) {
            inputEcoreFile = null;
            return;
        }

        inputEcoreFile = in;

        if (outputOWLFile == null) {
            int p = inputEcoreFile.lastIndexOf(".");
            outputOWLFile = (p != -1 ? inputEcoreFile.substring(0, p + 1)
                                       + "owl" : inputEcoreFile + ".owl");
        }

    }

    /**
     * set output file URL
     * 
     * @param out
     *            output file's URL
     */
    public void setOutputOWLFile(String out) {

        if (out == null)
            return;
        outputOWLFile = out;

    }

    /**
     * @return
     */
    public String getInputEcoreFile() {
        return inputEcoreFile;
    }

    /**
     * @return
     */
    public String getOutputOWLFile() {
        return outputOWLFile;
    }

    private Namespace getEObjectNS(EObject eobj) {
        Namespace ns = null;

        EObject nsOwner = eobj;
        while (nsOwner != null) {
            if (nsOwner instanceof EPackage) {
                ns = addNameSpace(((EPackage) nsOwner).getNsPrefix(),
                        ((EPackage) nsOwner).getNsURI());
                break;
            } else {
                nsOwner = nsOwner.eContainer();
            }
        }

        return (ns!=null) ? ns : dNS;
    }

    /**
     * 
     * @param classifier
     *            classifier to be transformed
     * @param ns
     *            namespace of the transformed class
     */
    private void transformEClass(EClassifier classifier) {
        PlainLiteral label;

        // EClass -> OWLClass
        if (classifier instanceof EClass) {
            EClass eclass = (EClass) classifier;

            OWLClass oclass = factory.createOWLClass();

            oclass.setLocalName(getUniquName(oclass, eclass.getName()));

            // use namespace of eContainer

            oclass.setNamespace(getEObjectNS(eclass));

            label = RDFSFactory.eINSTANCE.createPlainLiteral();
            label.setLexicalForm("EClass_" + eclass.getName());
            oclass.getRDFSLabel().add(label);
            createComments(ontology, eclass, oclass);

            ontology.getContains().add(oclass);
            addResource(eclass, oclass);
            addResource(oclass.getLocalName(), oclass);

        } else if (classifier instanceof EEnum) {
            // EEnum -> OWLOneOf
            EEnum eenum = (EEnum) classifier;
            EnumeratedClass enumclass = factory.createEnumeratedClass(ontology,
                    null);

            // enumclass.setLocalName("E-" + eenum.getName());
            // TODO1
            enumclass.setLocalName(getUniquName(enumclass, eenum.getName()));
            enumclass.setNamespace(getEObjectNS(eenum));

            label = RDFSFactory.eINSTANCE.createPlainLiteral();
            label.setLexicalForm("EEnum_" + eenum.getName());
            enumclass.getRDFSLabel().add(label);
            createComments(ontology, eenum, enumclass);

            // log
            sb.append("EEnum "
                      + eenum.getName()
                      + " transformed to OWLEnumeratedClass successfully!\n");

            // EEnumLiteral -> OWLMember
            Iterator et = eenum.getELiterals().iterator();
            while (et.hasNext()) {
                EEnumLiteral enumLiteral = (EEnumLiteral) et.next();
                RDFSLiteral literal = RDFSFactory.eINSTANCE
                        .createPlainLiteral();
                literal.setLexicalForm(enumLiteral.getName());
                Individual individual = factory.createIndividual(ontology,
                        enumLiteral.getName(), enumclass);

                individual.setLocalName(getUniquName(individual, enumLiteral
                        .getName()));
                individual.setNamespace(getEObjectNS(enumLiteral));

                enumclass.getOWLOneOf().add(individual);

                addResource(enumLiteral, individual);
                addResource(individual.getLocalName(), individual);
                // log
                // sb.append(" EEnumLiteral " + enumLiteral.getName() + "
                // transformed to individual successfully!\n");
            }

            ontology.getContains().add(enumclass);
            addResource(eenum, enumclass);
            addResource(enumclass.getLocalName(), enumclass);
        } else if (classifier instanceof EDataType) {
            // EDataType -> RDFSDatatype
            EDataType datatype = (EDataType) classifier;
            RDFSDatatype rdatatype = RDFSFactory.eINSTANCE.createRDFSDatatype();

            rdatatype.setLocalName(getUniquName(rdatatype, datatype.getName()));
            rdatatype.setNamespace(getEObjectNS(datatype));

            label = RDFSFactory.eINSTANCE.createPlainLiteral();
            label.setLexicalForm("EDatatype_" + datatype.getName());
            rdatatype.getRDFSLabel().add(label);
            createComments(ontology, datatype, rdatatype);

            ontology.getContains().add(rdatatype);
            addResource(datatype, rdatatype);
            addResource(rdatatype.getLocalName(), rdatatype);

        }

    }

    private MinCardinalityRestriction createMinCardinarlityRestriction(
            OWLObjectProperty property, int bound) {
        // cardinality
        MinCardinalityRestriction minCr = factory
                .createMinCardinalityRestriction();
        minCr.setNamespace(aNS);

        RDFSLiteral rdfsLiteral = RDFSFactory.eINSTANCE.createRDFSLiteral();
        rdfsLiteral.setLexicalForm(Integer.toString(bound));
        ontology.getContains().add(rdfsLiteral);

        minCr.setOWLOnProperty(property);
        minCr.setOWLMinCardinality(rdfsLiteral);
        ontology.getContains().add(minCr);
        return minCr;
    }

    private MaxCardinalityRestriction createMaxCardinarlityRestriction(
            OWLObjectProperty property, int bound) {
        // cardinality
        MaxCardinalityRestriction maxCr = factory
                .createMaxCardinalityRestriction();
        maxCr.setNamespace(aNS);

        RDFSLiteral rdfsLiteral = RDFSFactory.eINSTANCE.createRDFSLiteral();
        rdfsLiteral.setLexicalForm(Integer.toString(bound));
        ontology.getContains().add(rdfsLiteral);

        maxCr.setOWLOnProperty(property);
        maxCr.setOWLMaxCardinality(rdfsLiteral);
        ontology.getContains().add(maxCr);
        return maxCr;
    }

    private CardinalityRestriction createCardinarlityRestriction(
            OWLObjectProperty property, int bound) {
        // cardinality
        CardinalityRestriction cr = factory.createCardinalityRestriction();
        cr.setNamespace(aNS);

        RDFSLiteral rdfsLiteral = RDFSFactory.eINSTANCE.createRDFSLiteral();
        rdfsLiteral.setLexicalForm(Integer.toString(bound));
        ontology.getContains().add(rdfsLiteral);

        cr.setOWLOnProperty(property);
        cr.setOWLCardinality(rdfsLiteral);
        ontology.getContains().add(cr);
        return cr;
    }

    private HashMap resMap = new HashMap();

    private void addResource(Object key, RDFSResource val) {
        resMap.put(key, val);
    }

    private RDFSResource getResource(Object key) {
        return (RDFSResource) resMap.get(key);
    }

    private HashMap nsMap = new HashMap();

    private Namespace addNameSpace(String nsPrefix, String nsURI) {
        // add namespace
        Namespace ns = (Namespace) nsMap.get(nsPrefix);

        if (ns == null) {
            ns = RDFSFactory.eINSTANCE.createNamespace();
            ns.setName(nsPrefix);
            String URI = (nsURI.endsWith("#")) ? nsURI : nsURI + "#";
            ns.setURI(URI);
            ontology.getOwnedNamespace().add(ns);
            nsMap.put(nsPrefix, ns);
        }

        return ns;
    }

    public static void main(String[] args) {
        ECore2OWLTransformer ecore2owl = new ECore2OWLTransformer("./Ecore2OWLTransformer.log");
        try {
            ecore2owl.convertECore2OWL("testcases/GenericMetadata.ecore");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
