/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFParserResultHandler.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.rdf.resource.parser.wrapper;

import org.eclipse.eodm.rdf.resource.parser.NamespaceHandler;
import org.eclipse.eodm.rdf.resource.parser.StatementHandler;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;

/**
 * RDFParserResultHandler is an interface implementation to get RDF statements
 */
public class RDFParserResultHandler implements StatementHandler,
        NamespaceHandler {

    RDFTripleAnalyser analyser = null;

    public RDFParserResultHandler(RDFTripleAnalyser analyser) {
        if (analyser == null)
            throw new IllegalArgumentException(
                    "The RDF triple analyser can not be null.");

        this.analyser = analyser;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.rdfs.resource.parser.StatementHandler#exportStatement(org.eclipse.rdfs.resource.parser.element.RDFResourceElement,
     *      org.eclipse.rdfs.resource.parser.element.URIReference,
     *      org.eclipse.rdfs.resource.parser.element.RDFValue)
     */
    public void exportStatement(RDFResourceElement subject,
            URIReference predicate, RDFValue object) {
        analyser.analyseTriple(subject, predicate, object);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.rdfs.resource.parser.StatementHandler#exportReification(org.eclipse.rdfs.resource.parser.element.RDFResourceElement,
     *      org.eclipse.rdfs.resource.parser.element.RDFTriple)
     */
    public void exportReification(RDFResourceElement reifyNode,
            RDFResourceElement subject, URIReference predicate, RDFValue object) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.rdfs.resource.parser.StatementHandler#setSeparateExport(boolean)
     */
    public void setSeparateExport(boolean exportSeparate) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.rdfs.resource.parser.StatementHandler#isSeparateExport()
     */
    public boolean isSeparateExport() {
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.rdfs.resource.parser.NamespaceHandler#exportNamespace(java.lang.String,
     *      java.lang.String)
     */
    public void exportNamespace(String abbr, String fullURI) {
        analyser.handleNamespace(abbr, fullURI);
    }
}