/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved.   This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 *
 * $Id: RDFSClass.java,v 1.1.1.1 2006/01/31 09:13:24 gxie Exp $
 */
package org.eclipse.eodm.rdfs;

import org.eclipse.emf.common.util.EList;

/**
 * <!-- begin-user-doc --> A representation of the model object '
 * <em><b>RDFS Class</b></em>'. <!-- end-user-doc -->
 * 
 * <p>
 * The following features are supported:
 * <ul>
 * <li>
 * {@link org.eclipse.eodm.rdfs.RDFSClass#getRefByAVFRestriction <em>Ref By AVF Restriction</em>}
 * </li>
 * <li>
 * {@link org.eclipse.eodm.rdfs.RDFSClass#getRefBySVFRestriction <em>Ref By SVF Restriction</em>}
 * </li>
 * <li>
 * {@link org.eclipse.eodm.rdfs.RDFSClass#getRDFSSubClassOf <em>RDFS Sub Class Of</em>}
 * </li>
 * <li>{@link org.eclipse.eodm.rdfs.RDFSClass#getSubClass <em>Sub Class</em>}</li>
 * <li>
 * {@link org.eclipse.eodm.rdfs.RDFSClass#getOwnedProperty <em>Owned Property</em>}
 * </li>
 * <li>
 * {@link org.eclipse.eodm.rdfs.RDFSClass#getRangedByProperty <em>Ranged By Property</em>}
 * </li>
 * <li>{@link org.eclipse.eodm.rdfs.RDFSClass#getInstance <em>Instance</em>}</li>
 * </ul>
 * </p>
 * 
 * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass()
 * @model
 * @generated
 */
public interface RDFSClass extends RDFSResource {
    /**
     * Returns the value of the '<em><b>Ref By AVF Restriction</b></em>'
     * reference list. The list contents are of type
     * {@link org.eclipse.eodm.owl.AllValuesFromRestriction}. It is bidirectional
     * and its opposite is '
     * {@link org.eclipse.eodm.owl.AllValuesFromRestriction#getOWLAllValuesFrom <em>OWL All Values From</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>Ref By AVF Restriction</em>' reference
     *         list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_RefByAVFRestriction()
     * @see org.eclipse.eodm.owl.AllValuesFromRestriction#getOWLAllValuesFrom
     * @model type="org.eclipse.owl.AllValuesFromRestriction"
     *        opposite="OWLAllValuesFrom"
     * @generated
     */
    EList getRefByAVFRestriction();

    /**
     * Returns the value of the '<em><b>Ref By SVF Restriction</b></em>'
     * reference list. The list contents are of type
     * {@link org.eclipse.eodm.owl.SomeValuesFromRestriction}. It is bidirectional
     * and its opposite is '
     * {@link org.eclipse.eodm.owl.SomeValuesFromRestriction#getOWLSomeValuesFrom <em>OWL Some Values From</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>Ref By SVF Restriction</em>' reference
     *         list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_RefBySVFRestriction()
     * @see org.eclipse.eodm.owl.SomeValuesFromRestriction#getOWLSomeValuesFrom
     * @model type="org.eclipse.owl.SomeValuesFromRestriction"
     *        opposite="OWLSomeValuesFrom"
     * @generated
     */
    EList getRefBySVFRestriction();

    /**
     * Returns the value of the '<em><b>RDFS Sub Class Of</b></em>'
     * reference list. The list contents are of type
     * {@link org.eclipse.eodm.rdfs.RDFSClass}. It is bidirectional and its opposite
     * is '{@link org.eclipse.eodm.rdfs.RDFSClass#getSubClass <em>Sub Class</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>RDFS Sub Class Of</em>' reference list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_RDFSSubClassOf()
     * @see org.eclipse.eodm.rdfs.RDFSClass#getSubClass
     * @model type="org.eclipse.rdfs.RDFSClass" opposite="subClass"
     * @generated
     */
    EList getRDFSSubClassOf();

    /**
     * Returns the value of the '<em><b>RDFS Sub Class Of</b></em>'
     * reference list. The list contents are of type
     * {@link org.eclipse.eodm.rdfs.RDFSClass}. It is bidirectional and its opposite
     * is '{@link org.eclipse.eodm.rdfs.RDFSClass#getSubClass <em>Sub Class</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @param if
     *            bInference is true, it will give an answer after inference
     * @return the value of the '<em>RDFS Sub Class Of</em>' reference list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_RDFSSubClassOf()
     * @see org.eclipse.eodm.rdfs.RDFSClass#getSubClass
     * @model type="org.eclipse.rdfs.RDFSClass" opposite="subClass"
     */
    EList getRDFSSubClassOf(boolean bInference);

    /**
     * Returns the value of the '<em><b>Sub Class</b></em>' reference list.
     * The list contents are of type {@link org.eclipse.eodm.rdfs.RDFSClass}. It is
     * bidirectional and its opposite is '
     * {@link org.eclipse.eodm.rdfs.RDFSClass#getRDFSSubClassOf <em>RDFS Sub Class Of</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>Sub Class</em>' reference list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_SubClass()
     * @see org.eclipse.eodm.rdfs.RDFSClass#getRDFSSubClassOf
     * @model type="org.eclipse.rdfs.RDFSClass" opposite="RDFSSubClassOf"
     * @generated
     */
    EList getSubClass();

    /**
     * Returns the value of the '<em><b>Sub Class</b></em>' reference list.
     * The list contents are of type {@link org.eclipse.eodm.rdfs.RDFSClass}. It is
     * bidirectional and its opposite is '
     * {@link org.eclipse.eodm.rdfs.RDFSClass#getRDFSSubClassOf <em>RDFS Sub Class Of</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @param if
     *            bInference is true, it will give an answer after inference
     * @return the value of the '<em>Sub Class</em>' reference list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_SubClass()
     * @see org.eclipse.eodm.rdfs.RDFSClass#getRDFSSubClassOf
     * @model type="org.eclipse.rdfs.RDFSClass" opposite="RDFSSubClassOf"
     */
    EList getSubClass(boolean bInference);

    /**
     * Returns the value of the '<em><b>Owned Property</b></em>' reference
     * list. The list contents are of type {@link org.eclipse.eodm.rdfs.RDFProperty}.
     * It is bidirectional and its opposite is '
     * {@link org.eclipse.eodm.rdfs.RDFProperty#getRDFSDomain <em>RDFS Domain</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>Owned Property</em>' reference list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_OwnedProperty()
     * @see org.eclipse.eodm.rdfs.RDFProperty#getRDFSDomain
     * @model type="org.eclipse.rdfs.RDFProperty" opposite="RDFSDomain"
     * @generated
     */
    EList getOwnedProperty();

    /**
     * Returns the value of the '<em><b>Ranged By Property</b></em>'
     * reference list. The list contents are of type
     * {@link org.eclipse.eodm.rdfs.RDFProperty}. It is bidirectional and its
     * opposite is '
     * {@link org.eclipse.eodm.rdfs.RDFProperty#getRDFSRange <em>RDFS Range</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>Ranged By Property</em>' reference
     *         list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_RangedByProperty()
     * @see org.eclipse.eodm.rdfs.RDFProperty#getRDFSRange
     * @model type="org.eclipse.rdfs.RDFProperty" opposite="RDFSRange"
     * @generated
     */
    EList getRangedByProperty();

    /**
     * Returns the value of the '<em><b>Instance</b></em>' reference list.
     * The list contents are of type {@link org.eclipse.eodm.rdfs.RDFSResource}. It
     * is bidirectional and its opposite is '
     * {@link org.eclipse.eodm.rdfs.RDFSResource#getRDFType <em>RDF Type</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>Instance</em>' reference list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_Instance()
     * @see org.eclipse.eodm.rdfs.RDFSResource#getRDFType
     * @model type="org.eclipse.rdfs.RDFSResource" opposite="RDFType"
     * @generated
     */
    EList getInstance();

    /**
     * Returns the value of the '<em><b>Instance</b></em>' reference list.
     * The list contents are of type {@link org.eclipse.eodm.rdfs.RDFSResource}. It
     * is bidirectional and its opposite is '
     * {@link org.eclipse.eodm.rdfs.RDFSResource#getRDFType <em>RDF Type</em>}'.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @param if
     *            bInference is true, it will give an answer after inference
     * @return the value of the '<em>Instance</em>' reference list.
     * @see org.eclipse.eodm.rdfs.RDFSPackage#getRDFSClass_Instance()
     * @see org.eclipse.eodm.rdfs.RDFSResource#getRDFType
     * @model type="org.eclipse.rdfs.RDFSResource" opposite="RDFType"
     */
    EList getInstance(boolean bInference);

} // RDFSClass
